/*
* @Author: Qiaosen Huang
* @Date:   2016-12-27 15:31:39
* @Last Modified by:   Qiaosen Huang
* @Last Modified time: 2016-12-30 10:40:39
*/

'use strict';
const fs = require('fs');
const path = require('path');

function countLevel(str) {
    let spaces = 0;
    for (const char of str) {
        if (char === ' ') {
            spaces++;
        } else {
            break;
        }
    }
    return spaces / 4;
}



module.exports = function (contentDir) {
    function read(file) {
        return fs.readFileSync(path.resolve(contentDir, file)).toString();
    }
    function write(file, content) {
        return fs.writeFileSync(path.resolve(contentDir, file), content);
    }
    const summary = read('SUMMARY.md');

    const tree = {
        name: 'root',
        children: [],
    };
    let pointer = tree;

    const lines = summary.split('\n').slice(2);
    for (const line of lines) {
        if (line.trim().length === 0) break;
        const level = countLevel(line);
        const groups = line.match(/\[((?!(\\\]|\])).+)\]\(((?!(\\\)|\))).+)\)/)
        if (!groups) continue;
        const name = groups[1];
        const file = groups[3];
        if (file === 'README.md') continue;

        const item = {
            name,
            file,
            level,
            children: [],
        }

        if (!pointer.parent) { // top level
            item.parent = pointer,
            pointer.children.push(item);
            pointer = item;
            continue;
        }

        if  (level === pointer.level) { // sibling
            item.parent = pointer.parent;
            pointer.parent.children.push(item);
            pointer = item;
        } else if (level > pointer.level) { // children
            item.parent = pointer;
            pointer.children.push(item);
            pointer = item;
        } else if (level < pointer.level) { // step out
            while (level !== pointer.level) {
                pointer = pointer.parent;
            }
            item.parent = pointer.parent;
            pointer.parent.children.push(item);
            pointer = item;
        }
    }

    function append(item) {
        if (!item.children.length) return;
        let menu = '';
        const parentFile = read(item.file);
        for (const child of item.children) {
            menu += `<a href="${child.file.replace(/\.md$/, '.html')}" class="list-group-item">\n`;
            menu += `<i class="fa fa-file-0" aria-hidden="true"></i>${child.name}\n`;
            menu += '</a>\n';
            append(child);
        }
        menu = '<div class="list-group">\n' + menu  +'</div>\n';
        write(item.file, menu + parentFile);
    }

    for (const item of tree.children) {
        append(item);
    }

}